class RadarrTooManyVariablesException(Exception):
    pass


class RadarrInvalidIdSupplied(Exception):
    pass


class RadarrInvalidApiKey(Exception):
    pass


class RadarrMovieNotFound(Exception):
    pass


class RadarrValidationException(Exception):
    pass
