class SonarrTooManyVariablesException(Exception):
    pass


class SonarrInvalidIdSupplied(Exception):
    pass


class SonarrInvalidApiKey(Exception):
    pass


class SonarrMovieNotFound(Exception):
    pass


class SonarrValidationException(Exception):
    pass
